/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus.type;

import java.awt.Point;
import java.awt.image.DataBuffer;
import java.awt.image.WritableRaster;
import java.util.List;
import nl.cloudfarming.client.isobus.model.Grid;
import nl.cloudfarming.client.isobus.model.TreatmentZone;
import org.geotools.coverage.CoverageFactoryFinder;
import org.geotools.coverage.grid.GridCoverage2D;
import org.geotools.coverage.grid.GridCoverageFactory;
import org.geotools.geometry.jts.ReferencedEnvelope;
import org.geotools.referencing.crs.DefaultGeographicCRS;
import org.opengis.geometry.BoundingBox;

/**
 * Raster factory for Isobus grids
 * 
 * To create a raster it needs a Grid object and a list of treatment zones
 * It will assume that the treatment zones are in the right order
 * It assumes that element 0 == the bottom left corner of the grid
 * and that element size() - 1 == the top right corner
 * @author Merijn Zengers
 */
public class RasterFactory {

    private RasterFactory() {
    }

    /**
     * create a raster from grid data Calculates a raster from the information
     * in grid Then puts the values from the treatmentZones in this raster Will
     * take the first element of treatmentZones as left top box in the raster
     * Will then work from left to right and top to bottom through the raster
     * for each box taking the next element from treatmentZones This requires
     * that grid.getMaximumColumn() * grid.getMaximumRow() ==
     * treatMentZones.size() if this is not true will throw an
     * assertionException
     *
     * @param grid the grid to create a raster for
     * @param treatmentZones a list of treatmentZones for this grid
     * It will assume that the treatment zones are in the right order
     * It assumes that element 0 == the bottom left corner of the grid
     * and that element size() -1  == the top right corner
     * @return raster the raster created
     */
    public static GridCoverage2D createRaster(Grid grid, List<TreatmentZone> treatmentZones) {
        assert (grid.getMaximumColumn() * grid.getMaximumRow() == treatmentZones.size());

        double minx = grid.getMinimumEastPosition().doubleValue();
        double maxx = grid.getMinimumEastPosition().doubleValue() + (grid.getCellEastSize() * grid.getMaximumColumn());
        double miny = grid.getMinimumNorthPosition().doubleValue() ;
        double maxy = grid.getMinimumNorthPosition().doubleValue() + (grid.getCellNorthSize() * grid.getMaximumRow());

        // in pixels
        Double width = new Long(grid.getMaximumColumn()).doubleValue();
        Double height = new Long(grid.getMaximumRow()).doubleValue();

        int w = width.intValue();
        int h = height.intValue();
        int bands = 1;
        Point location = new Point(0, 0);
        WritableRaster raster = WritableRaster.createBandedRaster(DataBuffer.TYPE_INT, w, h, bands, location);

        for (int ih = 0; ih < h; ih++) {
            int counter = w;
            for (int iw = 0; iw < w; iw++) {
                // Our raster starts at the left top and ends at right bottom
                // The list provided from the grid file starts at the left bottom and ends at the right top
                // It has to be inverted.
                int treatmentZoneIndex = (((h - ih) * w) - counter);
                Short treatMentZoneColor =  treatmentZones.get(treatmentZoneIndex).getTreatmentZoneColor() == null ? 0 : treatmentZones.get(treatmentZoneIndex).getTreatmentZoneColor();
                raster.setSample(iw, ih, 0, treatMentZoneColor);
                counter--;
            }
        }

        BoundingBox boundingBox = new ReferencedEnvelope(minx, maxx, miny, maxy, DefaultGeographicCRS.WGS84);


        GridCoverageFactory factory = CoverageFactoryFinder.getGridCoverageFactory(null);
        return factory.create("name", raster, boundingBox);
    }
}
